def Red():
    """Returns the hex code for the color red."""
    return 0xFF0000

def Green():
    """Returns the hex code for the color green."""
    return 0x00FF00

def Blue():
    """Returns the hex code for the color blue."""
    return 0x0000FF

def Purple():
    """Returns the hex code for the color purple."""
    return 0x800080

def Yellow():
    """Returns the hex code for the color yellow."""
    return 0xFFFF00

def Orange():
    """Returns the hex code for the color orange."""
    return 0xFFA500

def Black():
    """Returns the hex code for the color black."""
    return 0x000000

def White():
    """Returns the hex code for the color white."""
    return 0xFFFFFF

def Gray():
    """Returns the hex code for the color gray."""
    return 0x808080